@@ -35,9 +35,10 @@
       var perms = [[0,0,1], [1,2,2], [2,1,0]],
           x, xrow, elem, A, d, nhits, i, j, k, u, v, w, intersect, which, v0, v2, vx, reverse,
           face1 = [], face2 = [], normals = [],
-          nPlanes = obj.normals.length;
+          nPlanes = obj.normals.length, idx, center;
       obj.bbox = bbox;
       obj.vertices = [];
+      obj.centers = [];
       obj.initialized = false;
       for (elem = 0; elem < nPlanes; elem++) {
 //    Vertex Av = normal.getRecycled(elem);
@@ -78,9 +79,9 @@
                   }
                 }
                 if (which > i+1) {
-                  this.swap(x, i+1, which);
-                  this.swap(face1, i+1, which);
-                  this.swap(face2, i+1, which);
+                  rglwidgetClass.swap(x, i+1, which);
+                  rglwidgetClass.swap(face1, i+1, which);
+                  rglwidgetClass.swap(face2, i+1, which);
                 }
               }
             }
@@ -89,16 +90,23 @@
               v0 = [x[0][0] - x[1][0] , x[0][1] - x[1][1], x[0][2] - x[1][2]];
               v2 = [x[2][0] - x[1][0] , x[2][1] - x[1][1], x[2][2] - x[1][2]];
               /* cross-product */
-              vx = this.xprod(v0, v2);
-              reverse = this.dotprod(vx, A) > 0;
+              vx = rglwidgetClass.xprod(v0, v2);
+              reverse = rglwidgetClass.dotprod(vx, A) > 0;
 
               for (i=0; i<nhits-2; i++) {
                 obj.vertices.push(x[0]);
+                center = [];
+                for (k = 0; k<3; k++)
+                  center.push(x[0][k]/3);
                 normals.push(A);
                 for (j=1; j<3; j++) {
-                  obj.vertices.push(x[i + (reverse ? 3-j : j)]);
+                  idx = i + (reverse ? 3-j : j);
+                  obj.vertices.push(x[idx]);
+                  for (k=0; k<3; k++)
+                    center[k] += x[idx][k]/3;
                   normals.push(A);
                 }
+                obj.centers.push(center);
               }
             }
       }
@@ -194,17 +202,22 @@
       var gl = this.gl,
           clipcheck = 0,
           clipplaneids = subscene.clipplanes,
-          clip, i,j;
-      for (i=0; i < clipplaneids.length; i++) {
-        clip = this.getObj(clipplaneids[i]);
-        for (j=0; j < clip.offsets.length; j++) {
-          gl.uniform4fv(obj.clipLoc[clipcheck + j], clip.IMVClip[j]);
+          clip, i,j, n = this.countClipplanes(),
+          clipplanedata; 
+          
+      if (n > 0) {
+        clipplanedata = new Float32Array(4*n);
+        for (i=0; i < clipplaneids.length; i++) {
+          clip = this.getObj(clipplaneids[i]);
+          for (j=0; j < clip.offsets.length; j++) {
+            clipplanedata.set(clip.IMVClip[j], clipcheck);
+            clipcheck += 4;
+          }
         }
-        clipcheck += clip.offsets.length;
+      
+        // Leftovers are initialized to zero, which is fine
+        gl.uniform4fv(obj.clipLoc, clipplanedata);
       }
-      if (typeof obj.clipLoc !== "undefined")
-        for (i=clipcheck; i < obj.clipLoc.length; i++)
-          gl.uniform4f(obj.clipLoc[i], 0,0,0,0);
     };
     
     /**
@@ -213,25 +226,58 @@
      * @param { object } subscene - Subscene to work with
      */
     rglwidgetClass.prototype.doLighting = function(obj, subscene) {
-      var gl = this.gl, i, light;
-        gl.uniformMatrix4fv( obj.normMatLoc, false, new Float32Array(this.normMatrix.getAsArray()) );
-        gl.uniform3fv( obj.emissionLoc, obj.emission);
-        gl.uniform1f( obj.shininessLoc, obj.shininess);
-        for (i=0; i < subscene.lights.length; i++) {
-          light = this.getObj(subscene.lights[i]);
-          if (!light.initialized) this.initObj(light);
-          gl.uniform3fv( obj.ambientLoc[i], this.componentProduct(light.ambient, obj.ambient));
-          gl.uniform3fv( obj.specularLoc[i], this.componentProduct(light.specular, obj.specular));
-          gl.uniform3fv( obj.diffuseLoc[i], light.diffuse);
-          gl.uniform3fv( obj.lightDirLoc[i], light.lightDir);
-          gl.uniform1i( obj.viewpointLoc[i], light.viewpoint);
-          gl.uniform1i( obj.finiteLoc[i], light.finite);
+    var gl = this.gl, i, j, n, light,
+      ambient, specular, diffuse, lightDir, viewpoint, finite,
+      ambient0, specular0;
+
+      gl.uniform3fv( obj.emissionLoc, obj.emission);
+      gl.uniform1f( obj.shininessLoc, obj.shininess);
+      while ((typeof subscene.lights === "undefined" ||
+              subscene.lights.length === 0) && 
+             typeof subscene.parent !== "undefined")
+        subscene = this.getObj(subscene.parent);
+
+      if (typeof subscene.lights === "undefined")
+        return;
+        
+      n = subscene.lights.length;
+        
+      ambient = new Float32Array(3*n);
+      specular = new Float32Array(3*n);
+      diffuse = new Float32Array(3*n);
+      lightDir = new Float32Array(3*n);
+      viewpoint = new Int32Array(n);
+      finite = new Int32Array(n);
+          
+      for (i=0; i < n; i++) {
+        light = this.getObj(subscene.lights[i]);
+        if (!light.initialized) this.initObj(light);
+        ambient0 = this.componentProduct(light.ambient, obj.ambient);
+        specular0 = this.componentProduct(light.specular, obj.specular);
+        for (j=0; j < 3; j++) {
+          ambient[3*i + j] = ambient0[j];
+          specular[3*i + j] = specular0[j];
+          diffuse[3*i + j] = light.diffuse[j];
+          lightDir[3*i + j] = light.lightDir[j];
         }
-        for (i=subscene.lights.length; i < obj.nlights; i++) {
-          gl.uniform3f( obj.ambientLoc[i], 0,0,0);
-          gl.uniform3f( obj.specularLoc[i], 0,0,0);
-          gl.uniform3f( obj.diffuseLoc[i], 0,0,0);
+        viewpoint[i] = light.viewpoint;
+        finite[i] = light.finite;
+      }
+        
+      for (i = n; i < obj.nlights; i++) {
+        for (j = 0; j < 3; j++) {
+          ambient[3*i + j] = 0.0;
+          specular[3*i + j] = 0.0;
+          diffuse[3*i + j] = 0.0;
         }
+      }
+        
+      gl.uniform3fv( obj.ambientLoc, ambient);
+      gl.uniform3fv( obj.specularLoc, specular);
+      gl.uniform3fv( obj.diffuseLoc, diffuse);
+      gl.uniform3fv( obj.lightDirLoc, lightDir);
+      gl.uniform1iv( obj.viewpointLoc, viewpoint);
+      gl.uniform1iv( obj.finiteLoc, finite);
     };
     
     /**
@@ -265,15 +311,25 @@
         return false;
     };
     
+    /**
+     * Do code for vNormal
+     * @param { object } obj - Object to work with
+     */
+    rglwidgetClass.prototype.doNormMat = function(obj) {
+      var gl = this.gl;
+        
+      gl.uniformMatrix4fv( obj.normMatLoc, false, new Float32Array(this.normMatrix.getAsArray()) );
+    };
+    
     /**
      * Do code for textures
      * @param { object } obj - Object to work with
      */    
     rglwidgetClass.prototype.doTexture = function(obj) {
       var gl = this.gl, 
-          is_spheres = obj.type === "spheres";
+          is_sphere = obj.type === "sphere";
         gl.enableVertexAttribArray( obj.texLoc );
-        if (is_spheres)
+        if (is_sphere)
           gl.vertexAttribPointer(obj.texLoc, 2, gl.FLOAT, false, 4*this.sphere.vOffsets.stride, 4*this.sphere.vOffsets.tofs);
         else
           gl.vertexAttribPointer(obj.texLoc, 2, gl.FLOAT, false, 4*obj.vOffsets.stride, 4*obj.vOffsets.tofs);
@@ -303,26 +359,51 @@
      * @param { object } obj - Object to work with
      */    
     rglwidgetClass.prototype.doUserUniforms = function(obj) {
+      var gl = this.gl, attr;
       if (typeof obj.userUniforms !== "undefined") {
-        var gl = this.gl;
-      	for (var attr in obj.userUniformLocations) {
+      	for (attr in obj.userUniformLocations) {
       	  var loc = obj.userUniformLocations[attr];
       	  if (loc !== null) {
       	    var uniform = obj.userUniforms[attr];
-      	    if (typeof uniform.length === "undefined")
-      	      gl.uniform1f(loc, uniform);
-      	    else if (typeof uniform[0].length === "undefined") {
-      	      uniform = new Float32Array(uniform);
-      	      switch(uniform.length) {
-      	      	case 2: gl.uniform2fv(loc, uniform); break;
-      	      	case 3: gl.uniform3fv(loc, uniform); break;
-      	      	case 4: gl.uniform4fv(loc, uniform); break;
-      	      	default: console.warn("bad uniform length");
-      	      }
-      	    } else if (uniform.length === 4 && uniform[0].length === 4)
-      	      gl.uniformMatrix4fv(loc, false, new Float32Array(uniform.getAsArray()));
-      	    else
-      	      console.warn("unsupported uniform matrix");
+      	    if (typeof uniform !== "undefined") {
+      	      var dim = rglwidgetClass.arrayDim(uniform);
+      	      if (dim.length === 0)
+      	        gl.uniform1f(loc, uniform);
+      	      else if (dim.length === 1) {
+      	        uniform = new Float32Array(uniform);
+      	        switch(uniform.length) {
+      	      	  case 2: gl.uniform2fv(loc, uniform); break;
+      	      	  case 3: gl.uniform3fv(loc, uniform); break;
+      	      	  case 4: gl.uniform4fv(loc, uniform); break;
+      	      	  default: console.warn("bad uniform length");
+      	        }
+      	      } else if (dim.length === 2 && dim[0] === 4 && dim[1] === 4)
+      	        gl.uniformMatrix4fv(loc, false, new Float32Array(rglwidgetClass.flatten(uniform)));
+      	      else if (dim.length === 2) {
+      	        uniform = new Float32Array(rglwidgetClass.flatten(uniform));
+      	        switch(dim[[1]]) {
+      	          case 1: gl.uniform1fv(loc, uniform); break;
+      	          case 2: gl.uniform2fv(loc, uniform); break;
+      	          case 3: gl.uniform3fv(loc, uniform); break;
+      	          case 4: gl.uniform4fv(loc, uniform); break;
+      	          default: console.warn("bad uniform column count");
+      	        }
+      	      } else
+      	        console.warn("unsupported uniform shape");
+      	    }
+      	  }
+      	}
+      }
+      if (typeof obj.userTextures !== "undefined") {
+        var has_texture = rglwidgetClass.isSet(obj.flags, rglwidgetClass.f_has_texture),
+              texnum = has_texture - 1;
+        for (attr in obj.userTextures) {
+      	  var texture = obj.userTextures[attr];
+      	  if (texture.sampler !== null) {
+      	    texnum += 1;
+      	    gl.activeTexture(gl.TEXTURE0 + texnum);
+            gl.bindTexture(gl.TEXTURE_2D, texture.texture);
+            gl.uniform1i( texture.sampler, texnum);
       	  }
       	}
       }
@@ -338,7 +419,7 @@
       var gl = this.gl,
           f = obj.f[pass],
           type = obj.type,
-          fat_lines = this.isSet(obj.flags, this.f_fat_lines),
+          fat_lines = rglwidgetClass.isSet(obj.flags, rglwidgetClass.f_fat_lines),
           fnew, step;
       switch(type){
         case "points":
@@ -462,19 +543,9 @@
      */       
     rglwidgetClass.prototype.drawSimple = function(obj, subscene, context) {
       var 
-          flags = obj.flags,
+          fl,
+          is_transparent,
           type = obj.type,
-          is_lit = this.isSet(flags, this.f_is_lit),
-          has_texture = this.isSet(flags, this.f_has_texture),
-          is_transparent = this.isSet(flags, this.f_is_transparent),
-          fixed_size = this.isSet(flags, this.f_fixed_size),
-          fixed_quads = this.isSet(flags, this.f_fixed_quads),
-          is_lines = this.isSet(flags, this.f_is_lines),
-          fat_lines = this.isSet(flags, this.f_fat_lines),
-          is_twosided = this.isSet(flags, this.f_is_twosided),
-          has_fog = this.isSet(flags, this.f_has_fog),
-          has_normals = (typeof obj.normals !== "undefined") ||
-                        obj.type === "sphere",
           gl = this.gl || this.initGL(),
           count,
           pass, mode, pmode,
@@ -482,12 +553,16 @@
         
       if (!obj.initialized)
         this.initObj(obj);
+        
+      if (this.texturesLoading)
+        return[];
 
       count = obj.vertexCount;
       if (!count)
         return [];
     
-      is_transparent = is_transparent || obj.someHidden;
+      fl = obj.defFlags;
+      is_transparent = fl.is_transparent || obj.someHidden;
       
       if (is_transparent && this.opaquePass)
         return this.getPieces(context, obj.id, 0, obj);
@@ -507,10 +582,13 @@
 
       this.doClipping(obj, subscene);
 
-      if (is_lit)
+      if (fl.needs_vnormal)
+        this.doNormMat(obj);
+        
+      if (fl.is_lit)
         this.doLighting(obj, subscene);
 
-      if (has_fog)
+      if (fl.has_fog)
         this.doFog(obj, subscene);
 
       this.doUserAttributes(obj);
@@ -520,18 +598,17 @@
       gl.enableVertexAttribArray( this.posLoc );
       enabled.posLoc = true;
         
-      if (has_texture || obj.type === "text")
+      if (fl.has_texture || obj.type === "text")
         enabled.texLoc = this.doTexture(obj);
 
       enabled.colLoc = this.doColors(obj);
-      if (is_lit)
-        enabled.normLoc = this.doNormals(obj);
+      enabled.normLoc = this.doNormals(obj);
 
-      if (fixed_size) {
+      if (fl.fixed_size) {
         gl.uniform3f( obj.textScaleLoc, 0.75/this.vp.width, 0.75/this.vp.height, 1.0);
       }
       
-      if (fixed_quads) {
+      if (fl.fixed_quads) {
         gl.enableVertexAttribArray( obj.ofsLoc );
         enabled.ofsLoc = true;
         gl.vertexAttribPointer(obj.ofsLoc, 3, gl.FLOAT, false, 4*obj.vOffsets.stride, 4*obj.vOffsets.oofs);
@@ -542,11 +619,11 @@
         if (pmode === "culled")
           continue;
 
-      	mode = fat_lines && (is_lines || pmode === "lines") ? "TRIANGLES" : this.mode4type[type];
+      	mode = fl.fat_lines && (fl.is_lines || pmode === "lines") ? "TRIANGLES" : this.mode4type[type];
 
-      	if (is_twosided) {
+      	if (fl.is_twosided) {
       	  gl.uniform1i(obj.frontLoc, pass !== 0);
-      	  if (has_normals) {
+      	  if (fl.has_normals) {
       	    gl.uniformMatrix4fv(obj.invPrMatLoc, false, new Float32Array(this.invPrMatrix.getAsArray()));
       	  }
       	}
@@ -561,13 +638,13 @@
           count = obj.f[pass].length;
           gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, obj.f[pass], gl.STATIC_DRAW);
         }
-      	if (!is_lines && pmode === "lines" && !fat_lines) {
+      	if (!fl.is_lines && pmode === "lines" && !fl.fat_lines) {
           mode = "LINES";
         } else if (pmode === "points") {
           mode = "POINTS";
         }
                           
-        if ((is_lines || pmode === "lines") && fat_lines) {
+        if ((fl.is_lines || pmode === "lines") && fl.fat_lines) {
           gl.enableVertexAttribArray(obj.pointLoc);
           enabled.pointLoc = true;
           gl.vertexAttribPointer(obj.pointLoc, 2, gl.FLOAT, false, 4*obj.vOffsets.stride, 4*obj.vOffsets.pointofs);
@@ -617,7 +694,7 @@
 
     rglwidgetClass.prototype.drawSpheres = function(obj, subscene, context) {
       var flags = obj.flags,
-          is_transparent = this.isSet(flags, this.f_is_transparent),
+          is_transparent = rglwidgetClass.isSet(flags, rglwidgetClass.f_is_transparent),
           sphereMV, baseofs, ofs, sscale, i,
           count, nc, scount, scale, indices, sphereNorm,
           enabled = {}, drawing,
@@ -728,7 +805,7 @@
       var count = obj.offsets.length,
         IMVClip = [];
       for (var i=0; i < count; i++) {
-        IMVClip[i] = this.multMV(this.invMatrix, obj.vClipplane.slice(4*i, 4*(i+1)));
+        IMVClip[i] = rglwidgetClass.multMV(this.invMatrix, obj.vClipplane.slice(4*i, 4*(i+1)));
       }
       obj.IMVClip = IMVClip;
       return [];
@@ -769,11 +846,14 @@
      */
     rglwidgetClass.prototype.drawSprites = function(obj, subscene, context) {
       var flags = obj.flags,
-          is_transparent = this.isSet(flags, this.f_is_transparent),
-          sprites3d = this.isSet(flags, this.f_sprites_3d),
+          is_transparent = rglwidgetClass.isSet(flags, rglwidgetClass.f_is_transparent),
+          sprites3d = rglwidgetClass.isSet(flags, rglwidgetClass.f_sprites_3d),
+          fixed_size = rglwidgetClass.isSet(flags, rglwidgetClass.f_fixed_size),
+          rotating = rglwidgetClass.isSet(flags, rglwidgetClass.f_rotating),
           i,j,
           origMV = new CanvasMatrix4( this.mvMatrix ),
           origPRMV = null,
+          origPR,
           pos, radius, userMatrix,
           result = [], margin = obj.material.margin;
  
@@ -796,7 +876,6 @@
           savenorm = new CanvasMatrix4(this.normMatrix),
           iOrig, adj, offset;
 
-      this.normMatrix = subscene.spriteNormmat;
       userMatrix = obj.userMatrix;
                    
       if (this.opaquePass) {
@@ -809,22 +888,53 @@
          origPRMV = new CanvasMatrix4( this.prmvMatrix );
 
       offset = obj.offset;
+      
+      if (fixed_size && !rotating) {
+        origPR = this.prMatrix;
+        this.prMatrix = new CanvasMatrix4();
+      }
         
       for (iOrig=0; iOrig < norigs; iOrig++) {
         if (this.opaquePass)
           j = iOrig;
         else
           j = context.subid;
-
-        pos = this.multVM([].concat(obj.vertices[j]).concat(1.0),
-                          origMV);
+        pos = [].concat(obj.vertices[j]).concat(1.0);
         radius = obj.radii.length > 1 ? obj.radii[j][0] : obj.radii[0][0];
         this.mvMatrix = new CanvasMatrix4(userMatrix);
         adj = this.getAdj(obj, j, offset);
         this.mvMatrix.translate(1 - 2*adj[0], 1 - 2*adj[1], 1 - 2*adj[2]);
-        this.mvMatrix.scale(radius);
-        this.mvMatrix.translate(pos[0]/pos[3], pos[1]/pos[3], pos[2]/pos[3]);
+        this.mvMatrix.scale(radius, radius, radius);
+        
+        if (fixed_size) {
+          var viewport = subscene.par3d.viewport,
+            winwidth = viewport.width*this.canvas.width,
+            winheight = viewport.height*this.canvas.height,
+            scalex = 27/winwidth, scaley = 27/winheight,
+              scale = Math.sqrt(scalex * scaley);
+          if (!rotating) {
+            pos = rglwidgetClass.multVM(pos, origMV);
+            pos = rglwidgetClass.multVM(pos, origPR);
+            this.mvMatrix.scale(scalex, scaley, scale);
+          } else {
+            scale = 4.0 * scale * subscene.par3d.zoom;
+            this.mvMatrix.scale(scale, scale, scale);
+          }
+          this.mvMatrix.translate(pos[0]/pos[3], pos[1]/pos[3], pos[2]/pos[3]);
+          if (rotating)
+            this.mvMatrix.multRight(origMV);
+        } else {
+          if (!rotating) {
+            pos = rglwidgetClass.multVM(pos, origMV);
+            this.mvMatrix.translate(pos[0]/pos[3], pos[1]/pos[3], pos[2]/pos[3]);
+          } else {
+            this.mvMatrix.translate(pos[0]/pos[3], pos[1]/pos[3], pos[2]/pos[3]);
+            this.mvMatrix.multRight(origMV);
+          }
+        }
+        this.setnormMatrix2();
         this.setprmvMatrix();
+      
         for (i=0; i < obj.objects.length; i++)
           if (this.opaquePass)
             result = result.concat(this.drawObjId(obj.objects[i], subscene.id, context.concat(j)));
@@ -833,6 +943,8 @@
       }
       this.normMatrix = savenorm;
       this.mvMatrix = origMV;
+      if (fixed_size && !rotating)
+        this.prMatrix = origPR;
       if (origPRMV !== null)
         this.prmvMatrix = origPRMV;
       return result;
@@ -862,7 +974,7 @@
      */
     rglwidgetClass.prototype.drawBBox = function(obj, subscene, context) {
       var flags = obj.flags,
-          is_transparent = this.isSet(flags, this.f_is_transparent),
+          is_transparent = rglwidgetClass.isSet(flags, rglwidgetClass.f_is_transparent),
           scale, bbox, indices,
           enabled = {}, drawing,
           result = [], idx, center, edges,
@@ -913,29 +1025,33 @@
       } else 
         result = result.concat(this.getCubePieces(context, obj));
 
+      if (!obj.ticks.initialized) {
+        obj.ticks.locations = this.getTickLocations(obj);
+        obj.ticks.edges = undefined;
+      }
+      edges = this.getTickEdges(this.prmvMatrix);
+      if (obj.needsAxisCallback) 
+        this.doAxisCallback(obj, edges);
+      if (!obj.ticks.edges || edges.toString() !== obj.ticks.edges.toString()) {
+        obj.ticks.edges = edges;
+        this.getTickVertices(obj.ticks);
+        this.placeTickLabels(obj);
+        this.setTickLabels(obj);
+      }
+      if (!obj.ticks.initialized) {
+        this.initObj(obj.ticks);
+        this.initObj(obj.labels);
+      }
+        
       if (drawing) {
-        if (!obj.ticks.initialized) {
-          obj.ticks.locations = this.getTickLocations(obj);
-          obj.ticks.edges = undefined;
-        }
-        edges = this.getTickEdges(this.prmvMatrix);
-        if (obj.needsAxisCallback) 
-          this.doAxisCallback(obj, edges);
-        if (!obj.ticks.edges || edges.toString() !== obj.ticks.edges.toString()) {
-          obj.ticks.edges = edges;
-          this.getTickVertices(obj.ticks);
-          this.placeTickLabels(obj);
-          this.setTickLabels(obj);
-        }
-        if (!obj.ticks.initialized) {
-          this.initObj(obj.ticks);
-          this.initObj(obj.labels);
-        }
         this.drawSimple(obj.ticks, subscene, context);
         this.drawSimple(obj.labels, subscene, context);
-      }
-      if (drawing)
+
         this.disableArrays(obj, enabled);
+      } else {
+        result = result.concat(this.drawSimple(obj.ticks, subscene, context));
+        result = result.concat(this.drawSimple(obj.labels, subscene, context));
+      }
 
       this.restoreBBoxMatrices(saved);
         
@@ -996,27 +1112,35 @@
      * @param { number } id - id of background object
      * @param { number } subsceneid - id of subscene
      */
-    rglwidgetClass.prototype.drawBackground = function(id, subsceneid) {
+    rglwidgetClass.prototype.drawBackground = function(id, subsceneid, context) {
       var gl = this.gl || this.initGL(),
           obj = this.getObj(id),
-          bg, i, savepr, saveinvpr, savemv;
+          subscene,
+          bg, i, savepr, saveinvpr, savemv, savenorm, m, bbox, result = [], 
+          savedm = gl.getParameter(gl.DEPTH_WRITEMASK),
+          savedt = gl.isEnabled(gl.DEPTH_TEST),
+          saveblend = gl.isEnabled(gl.BLEND);
 
       if (!obj.initialized)
         this.initObj(obj);
 
       if (obj.colors.length) {
         bg = obj.colors[0];
-        gl.clearColor(bg[0], bg[1], bg[2], bg[3]);
         gl.depthMask(true);
-        /* jshint bitwise: false */
-        gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);
-        /* jshint bitwise: true */
+        gl.clear(gl.DEPTH_BUFFER_BIT);
+        gl.clearColor(bg[0], bg[1], bg[2], bg[3]);
+        gl.clear(gl.COLOR_BUFFER_BIT);
         this.fogColor = bg;
-      } else 
+      } else {
         this.fogColor = [0,0,0,0];
+        obj.colors = [[0,0,0,0]];
+      }
   
       this.fogType = obj.fogtype;
       this.fogScale = obj.fogscale;
+      gl.disable(gl.BLEND);
+      gl.disable(gl.DEPTH_TEST);
+      gl.depthMask(false);
       if (typeof obj.quad !== "undefined") {
         savepr = this.prMatrix;
         saveinvpr = this.invPrMatrix;
@@ -1024,15 +1148,60 @@
         this.prMatrix = new CanvasMatrix4();
         this.invPrMatrix = new CanvasMatrix4();
         this.mvMatrix = new CanvasMatrix4();
-        gl.disable(gl.BLEND);
-        gl.disable(gl.DEPTH_TEST);
-        gl.depthMask(false);
         for (i=0; i < obj.quad.length; i++)
-          this.drawObjId(obj.quad[i], subsceneid);
+          result = result.concat(this.drawObjId(obj.quad[i], subsceneid));
         this.prMatrix = savepr;
         this.invPrMatrix = saveinvpr;
         this.mvMatrix = savemv;
+
+      } else if (obj.sphere) {
+        subscene = this.getObj(subsceneid);
+        savemv = this.mvMatrix;
+        savenorm = this.normMatrix;
+        bbox = subscene.par3d.bbox;
+        var center = [(bbox[0] + bbox[1])/2, 
+                  (bbox[2] + bbox[3])/2, 
+                  (bbox[4] + bbox[5])/2, 1],
+            scale = subscene.par3d.scale,
+            ranges = [bbox[1] - bbox[0], 
+                  bbox[3] - bbox[2],
+                  bbox[5] - bbox[4]],
+            avgscale = rglwidgetClass.vlen(ranges)/Math.sqrt(3),
+            aspect = [ranges[0]*scale[0]/avgscale,
+                      ranges[1]*scale[1]/avgscale,
+                      ranges[2]*scale[2]/avgscale],
+            maxaspect = Math.max(aspect[0], aspect[1], aspect[2]),
+            zoom = subscene.par3d.zoom;
+        m = new CanvasMatrix4();
+        m.rotate(90, 1, 0, 0);
+        m.scale(zoom*2.0*maxaspect*ranges[0]/aspect[0], 
+                zoom*2.0*maxaspect*ranges[1]/aspect[1],
+                zoom*2.0*maxaspect*ranges[2]/aspect[2]);
+        m.translate(center[0], center[1], center[2]);
+        m.multRight(savemv);
+        center = rglwidgetClass.multVM(center, savemv);
+        m.translate(-center[0], -center[1], -center[2]);
+        m.scale(1, 1, 0.25/zoom);
+        m.translate(center[0], center[1], center[2]);
+        this.mvMatrix = m;
+        this.initShapeFromObj(this.sphere, obj);
+        this.sphere.onecolor = obj.colors.length > 1 ? obj.colors[1] : obj.colors[0];
+        
+        this.normMatrix = new CanvasMatrix4();
+        
+        this.setnormMatrix2();
+        this.setprmvMatrix();
+        
+        result = result.concat(this.drawSimple(this.sphere, subscene, context));
+        this.mvMatrix = savemv;
+        this.normMatrix = savenorm;
       }
+      gl.depthMask(savedm);
+      if (savedt)
+        gl.enable(gl.DEPTH_TEST);
+      if (saveblend)
+        gl.enable(gl.BLEND);
+      return result;
     };
 
     /**
@@ -1058,13 +1227,13 @@
           flags = obj.flags;
           if (typeof flags !== "undefined") {
             subscene_has_faces = subscene_has_faces || 
-                            (this.isSet(flags, this.f_is_lit) &&
-                            !this.isSet(flags, this.f_fixed_quads));
+                            (rglwidgetClass.isSet(flags, rglwidgetClass.f_is_lit) &&
+                            !rglwidgetClass.isSet(flags, rglwidgetClass.f_fixed_quads));
             obj.is_transparent = obj.someHidden || 
-              this.isSet(flags, this.f_is_transparent);
+              rglwidgetClass.isSet(flags, rglwidgetClass.f_is_transparent);
             subscene_needs_sorting = subscene_needs_sorting || 
               obj.is_transparent ||
-              this.isSet(flags, this.f_depth_sort);
+              rglwidgetClass.isSet(flags, rglwidgetClass.f_depth_sort);
           }
         }
       }
@@ -1075,36 +1244,31 @@
       this.setInvPrMatrix();
       this.setmvMatrix(subsceneid);
       this.setnormMatrix2();
+      this.setprmvMatrix();
+      this.invMatrix = new CanvasMatrix4(this.mvMatrix);
+      this.invMatrix.invert();
+      
+      if (this.opaquePass) {
+        context = context.slice();
+        context.push(subsceneid);
         
-      if (typeof sub.backgroundId !== "undefined" && this.opaquePass)
-        this.drawBackground(sub.backgroundId, subsceneid);
+        this.doBlending(false);
+        this.subsceneid = subsceneid;
+        if (typeof this.sphere !== "undefined") // reset this.sphere.fastpieces; it will be recreated if needed
+          this.sphere.fastpieces = undefined;
+        if (typeof sub.backgroundId !== "undefined")
+          result = result.concat(this.drawBackground(sub.backgroundId, subsceneid, context));
+      }
 
       if (subids.length) {
-        if (subscene_has_faces &&
-            this.isSet(sub.flags, this.f_sprites_3d) &&
-            typeof sub.spriteNormmat === "undefined") {
-          sub.spriteNormmat = new CanvasMatrix4(this.normMatrix);
-        }
-
-        if (subscene_needs_sorting)
-          this.setprmvMatrix();
             
         if (clipids.length > 0) {
-          this.invMatrix = new CanvasMatrix4(this.mvMatrix);
-          this.invMatrix.invert();
           for (i = 0; i < clipids.length; i++)
             this.drawObjId(clipids[i], subsceneid);
         }
         
         subids = sub.opaque.concat(sub.transparent);
         if (this.opaquePass) {
-          context = context.slice();
-          context.push(subsceneid);
-        
-          this.doBlending(false);
-          this.subsceneid = subsceneid;
-          if (typeof this.sphere !== "undefined") // reset this.sphere.fastpieces; it will be recreated if needed
-            this.sphere.fastpieces = undefined;
           for (i = 0; i < subids.length; i++)
             result = result.concat(this.drawObjId(subids[i], subsceneid, context));
           subids = sub.subscenes;
