@@ -321,10 +321,11 @@ void rgl::rgl_attrib_count(int* id, int* attrib, int* count)
     RGLView* rglview = device->getRGLView();
     Scene* scene = rglview->getScene();
     Subscene* subscene = scene->whichSubscene(*id);
-    AABox bbox = subscene->getBoundingBox();
     SceneNode* scenenode = scene->get_scenenode(*id);
+    // getBoundingBox is called for the side effect of possibly calculating data_bbox.
+    subscene->getBoundingBox();
     if ( scenenode )
-      *count = scenenode->getAttributeCount(bbox, *attrib);
+      *count = scenenode->getAttributeCount(subscene, *attrib);
     else
       *count = 0;
   }
@@ -342,10 +343,9 @@ void rgl::rgl_attrib(int* id, int* attrib, int* first, int* count, double* resul
     RGLView* rglview = device->getRGLView();
     Scene* scene = rglview->getScene();
     Subscene* subscene = scene->whichSubscene(*id);
-    AABox bbox = subscene->getBoundingBox();
     SceneNode* scenenode = scene->get_scenenode(*id);
     if ( scenenode )
-      scenenode->getAttribute(bbox, *attrib, *first, *count, result);
+      scenenode->getAttribute(subscene, *attrib, *first, *count, result);
   }
 } 
 
@@ -360,12 +360,12 @@ void rgl::rgl_text_attrib(int* id, int* attrib, int* first, int* count, char** r
   if (deviceManager && (device = deviceManager->getCurrentDevice())) {
     RGLView* rglview = device->getRGLView();
     Scene* scene = rglview->getScene();
-    AABox bbox = scene->getBoundingBox();
+    Subscene* subscene = scene->whichSubscene(*id);
     SceneNode* scenenode = scene->get_scenenode(*id);
     
     if (scenenode)
       for (int i=0; i < *count; i++) {
-        String s = scenenode->getTextAttribute(bbox, *attrib, i + *first);
+        String s = scenenode->getTextAttribute(subscene, *attrib, i + *first);
         if (s.length) {
           *result = R_alloc(s.length + 1, 1);
           strncpy(*result, s.text, s.length);
@@ -696,6 +696,7 @@ void rgl::rgl_sprites(int* successptr, int* idata, double* vertex,
     int nshapes = idata[2];
     bool fixedSize = (bool)idata[3];
     int npos = idata[4];
+    bool rotating = (bool)idata[5];
     int count = 0;
     Shape** shapelist;
     Scene* scene = NULL;
@@ -721,7 +722,7 @@ void rgl::rgl_sprites(int* successptr, int* idata, double* vertex,
     success = as_success( device->add( new SpriteSet(currentMaterial, nvertex, vertex, nradius, radius,
                      device->getIgnoreExtent() || currentMaterial.marginCoord >= 0, 
     						     count, shapelist, userMatrix,
-    						     fixedSize, scene, adj, npos, pos, *offset) ) );
+    						     fixedSize, rotating, scene, adj, npos, pos, *offset) ) );
     CHECKGLERROR;
   }
 
