@@ -37,7 +37,7 @@ void FPS::render(double t, RenderContext* ctx)
   glRasterPos2f( 1.0f, -0.9f);
 
   if (ctx->font)
-    ctx->font->draw(buffer, static_cast<int>(strlen(buffer)), -1, 0.0, 0, *ctx);
+    ctx->font->draw(buffer, static_cast<int>(strlen(buffer)), -1, 0.0, 0.5, 0, *ctx);
 
   framecnt++;
 #endif
